@@ -1,3 +1,5 @@
 from spatialmath.pose2d import *
 from spatialmath.pose3d import *
 from spatialmath.quaternion import *
+from spatialmath.super_pose import *
+from spatialmath import base
